@@ -13,7 +13,7 @@ class ClientSync(object):
     def __init__(self, client):
         self._client = client
 
-    def new_job(self, task, inputdata, launcher_name="Unknown", debug=False):
+    def new_job(self, priority, task, inputdata, launcher_name="Unknown", debug=False):
         """
             Runs a new job.
             It works exactly like the Client class, instead that there is no callback and directly returns result, in the form of a tuple
@@ -21,14 +21,14 @@ class ClientSync(object):
         """
         job_semaphore = threading.Semaphore(0)
 
-        def manage_output(result, grade, problems, tests, custom, archive, stdout, stderr):
+        def manage_output(result, grade, problems, tests, custom, state, archive, stdout, stderr):
             """ Manages the output of this job """
-            manage_output.job_return = (result, grade, problems, tests, custom, archive, stdout, stderr)
+            manage_output.job_return = (result, grade, problems, tests, custom, state, archive, stdout, stderr)
             job_semaphore.release()
 
         manage_output.job_return = None
 
-        self._client.new_job(task, inputdata, manage_output, launcher_name, debug)
+        self._client.new_job(priority, task, inputdata, manage_output, launcher_name, debug)
         job_semaphore.acquire()
         job_return = manage_output.job_return
         return job_return
