@@ -3,6 +3,7 @@ import abc
 from inginious.frontend.environment_types.docker import DockerEnvType
 from inginious.frontend.environment_types.kata import KataEnvType
 from inginious.frontend.environment_types.mcq import MCQEnvType
+from inginious.frontend.environment_types.nvidia import NvidiaEnvType
 
 __env_types = {}
 
@@ -24,7 +25,6 @@ def register_env_type(env_obj):
 def register_base_env_types():
     # register standard env types here
     register_env_type(DockerEnvType())
+    register_env_type(NvidiaEnvType())
     register_env_type(KataEnvType())
     register_env_type(MCQEnvType())
-
-
